// vertex attributes required
attribute vec4 re_Position;
attribute vec3 re_Normal;
attribute vec2 re_TexCoord0;

// vertex shader to fragment shader
varying vec2  vTexCoord;
varying vec4  vDiffuse;
varying vec4  vSpecular;

// scene parameters
uniform vec4 ambient;

// the surface parameters of the mesh
uniform float Kd;
uniform float Ks;
uniform float Ka;
uniform float n_specular;

void main(void) {   

	// Output transformed vertex position:
	gl_Position = re_ModelViewProjectionMatrix * re_Position;

	// Transform vertex position into view space:
	vec3 vPos =  vec3(re_ModelViewMatrix * re_Position);
	
	// Compute the light direction in view space:
	vec3 vLightDir = normalize(mat3(re_ViewMatrix) * re_SunLights[0].direction);
	
	// Transform normal into view space:
	vec3 vNormal = normalize( re_NormalMatrix * re_Normal);
	
	// Compute reflection vector in view space:
	vec3 vReflect = normalize( 2.0 * dot(vNormal, vLightDir) * vNormal - vLightDir );         

	// Compute view vector (view space):
	vec3 vecView = -normalize(vPos);                

	// Compute diffuse and ambient contribution: 
	vDiffuse = ambient * Ka + re_SunLights[0].color * Kd * max(0.0, dot(vNormal, vLightDir));

	// Compute specular term:
	vSpecular = re_SunLights[0].color * Ks * pow(max(0.0, dot(vReflect, vecView)), n_specular);    

	vTexCoord = re_TexCoord0;
}
