/****************************************************************************
Copyright 2010 Adil Yalcin

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at 
http://www.apache.org/licenses/LICENSE-2.0 

Unless required by applicable law or agreed to in writing, software 
distributed under the License is distributed on an "AS IS" BASIS, 
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
See the License for the specific language governing permissions and 
limitations under the License. 
****************************************************************************/
#include "Phy/RigidBodyBox.h"
#include "Phy/Simulator.h"

#include "Phy/Space.h"
#include "Phy/Geom.h"

namespace Phy{

RigidBodyBox::RigidBodyBox(World& w, float density, const REng::Vector3 &size)
	:RigidBody(w)
{
	mSubstance.setBox(density,size);
	(new GeomBox(w.getDefaultSpace(),size))->attachToBody(*this);
}

RigidBodyBox::~RigidBodyBox() {
	//both geom and body are destroyed in the super class destructor
}

//////////////////////////////////////////////////////////////////////////
// GETTERS
REng::Vector3 RigidBodyBox::getSize() const {
	const Geom* g=getGeom();
	assert(g->getType()==GeomTypeBox);
	const GeomBox* gb=(const GeomBox*)g;
	return gb->getSideLengths();
}

} // namespace Phy
